@@ -1,14 +1,17 @@
+use super::browser_name::BrowserNameAtom;
+use ahash::AHashMap;
 use once_cell::sync::Lazy;
-use regex::Regex;
 use serde::Deserialize;
-use std::{borrow::Cow, collections::HashMap};
-use ustr::{Ustr, UstrMap};
+use std::borrow::Cow;
+
+pub(crate) mod features;
+pub(crate) mod region;
 
 pub const ANDROID_EVERGREEN_FIRST: f32 = 37.0;
 
 #[derive(Clone, Debug, Deserialize)]
 pub struct BrowserStat {
-    name: Ustr,
+    name: BrowserNameAtom,
     pub version_list: Vec<VersionDetail>,
 }
 
@@ -19,7 +22,7 @@ pub struct VersionDetail {
     pub release_date: Option<i64>,
 }
 
-pub type CaniuseData = UstrMap<BrowserStat>;
+pub type CaniuseData = AHashMap<BrowserNameAtom, BrowserStat>;
 
 pub static CANIUSE_BROWSERS: Lazy<CaniuseData> = Lazy::new(|| {
     serde_json::from_str(include_str!(concat!(
@@ -29,65 +32,67 @@ pub static CANIUSE_BROWSERS: Lazy<CaniuseData> = Lazy::new(|| {
     .unwrap()
 });
 
-pub static CANIUSE_USAGE: Lazy<Vec<(Ustr, String, f32)>> = Lazy::new(|| {
+pub static CANIUSE_GLOBAL_USAGE: Lazy<Vec<(BrowserNameAtom, String, f32)>> = Lazy::new(|| {
     serde_json::from_str(include_str!(concat!(
         env!("OUT_DIR"),
-        "/caniuse-usage.json"
+        "/caniuse-global-usage.json"
     )))
     .unwrap()
 });
 
-pub static BROWSER_VERSION_ALIASES: Lazy<UstrMap<HashMap<&'static str, &'static str>>> =
-    Lazy::new(|| {
-        let mut aliases = CANIUSE_BROWSERS
-            .iter()
-            .filter_map(|(name, stat)| {
-                let aliases = stat
-                    .version_list
-                    .iter()
-                    .filter_map(|version| {
-                        version
-                            .version
-                            .split_once('-')
-                            .map(|(bottom, top)| (bottom, top, version.version.as_str()))
-                    })
-                    .fold(
-                        HashMap::<&str, &str>::new(),
-                        move |mut aliases, (bottom, top, version)| {
-                            let _ = aliases.insert(bottom, version);
-                            let _ = aliases.insert(top, version);
-                            aliases
-                        },
-                    );
-                if aliases.is_empty() {
-                    None
-                } else {
-                    Some((*name, aliases))
-                }
-            })
-            .collect::<UstrMap<_>>();
-        let _ = aliases.insert(Ustr::from("op_mob"), {
-            let mut aliases = HashMap::new();
-            let _ = aliases.insert("59", "58");
-            aliases
-        });
+pub static BROWSER_VERSION_ALIASES: Lazy<
+    AHashMap<BrowserNameAtom, AHashMap<&'static str, &'static str>>,
+> = Lazy::new(|| {
+    let mut aliases = CANIUSE_BROWSERS
+        .iter()
+        .filter_map(|(name, stat)| {
+            let aliases = stat
+                .version_list
+                .iter()
+                .filter_map(|version| {
+                    version
+                        .version
+                        .split_once('-')
+                        .map(|(bottom, top)| (bottom, top, version.version.as_str()))
+                })
+                .fold(
+                    AHashMap::<&str, &str>::new(),
+                    move |mut aliases, (bottom, top, version)| {
+                        let _ = aliases.insert(bottom, version);
+                        let _ = aliases.insert(top, version);
+                        aliases
+                    },
+                );
+            if aliases.is_empty() {
+                None
+            } else {
+                Some((name.clone(), aliases))
+            }
+        })
+        .collect::<AHashMap<BrowserNameAtom, _>>();
+    let _ = aliases.insert("op_mob".into(), {
+        let mut aliases = AHashMap::new();
+        let _ = aliases.insert("59", "58");
         aliases
     });
-
-static REGEX_NON_DESKTOP_ANDROID: Lazy<Regex> =
-    Lazy::new(|| Regex::new(r"^(?:[2-4]\.|[34]$)").unwrap());
+    aliases
+});
 
 static ANDROID_TO_DESKTOP: Lazy<BrowserStat> = Lazy::new(|| {
-    let chrome = CANIUSE_BROWSERS.get(&Ustr::from("chrome")).unwrap();
-    let mut android = CANIUSE_BROWSERS
-        .get(&Ustr::from("android"))
-        .unwrap()
-        .clone();
+    let chrome = CANIUSE_BROWSERS.get(&"chrome".into()).unwrap();
+    let mut android = CANIUSE_BROWSERS.get(&"android".into()).unwrap().clone();
 
     android.version_list = android
         .version_list
         .into_iter()
-        .filter(|version| REGEX_NON_DESKTOP_ANDROID.is_match(&version.version))
+        .filter(|version| {
+            let version = &version.version;
+            version.starts_with("2.")
+                || version.starts_with("3.")
+                || version.starts_with("4.")
+                || version == "3"
+                || version == "4"
+        })
         .chain(
             chrome.version_list.iter().cloned().skip(
                 chrome.version_list.len()
@@ -108,7 +113,7 @@ static ANDROID_TO_DESKTOP: Lazy<BrowserStat> = Lazy::new(|| {
 });
 
 static OPERA_MOBILE_TO_DESKTOP: Lazy<BrowserStat> = Lazy::new(|| {
-    let mut op_mob = CANIUSE_BROWSERS.get(&Ustr::from("opera")).unwrap().clone();
+    let mut op_mob = CANIUSE_BROWSERS.get(&"opera".into()).unwrap().clone();
 
     if let Some(v) = op_mob
         .version_list
@@ -138,17 +143,17 @@ pub fn get_browser_stat(
                 "android" => Some(("android", &ANDROID_TO_DESKTOP)),
                 "op_mob" => Some(("op_mob", &OPERA_MOBILE_TO_DESKTOP)),
                 _ => CANIUSE_BROWSERS
-                    .get(&Ustr::from(desktop_name))
+                    .get(&desktop_name.into())
                     .map(|stat| (get_mobile_by_desktop_name(desktop_name), stat)),
             }
         } else {
             CANIUSE_BROWSERS
-                .get(&Ustr::from(name))
+                .get(&name.into())
                 .map(|stat| (&*stat.name, stat))
         }
     } else {
         CANIUSE_BROWSERS
-            .get(&Ustr::from(name))
+            .get(&name.into())
             .map(|stat| (&*stat.name, stat))
     }
 }
